/**********************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the Modelica License 2
* Copyright © 2013, Victorino Sanz.
* 
************************************************************/

#ifndef CELLULARAUTOMATA
#define CELLULARAUTOMATA

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#ifdef _WIN32
      #include <windows.h>
#endif

/*****************************************/
/* 1D CELLULAR AUTOMATA                  */
/*****************************************/


/* Object data type */
typedef struct Cell1D{
    void* cellstate; // state of cell
    int pos; // position in the space (column in 1D)
    int active; // boolean flag to set cell as active or not
    //struct Cell1D** neighbors; // pointers to neighbors
    int *neighbors_present; // boolean flag that indicates if neighbor exists or if it is outside space boundaries.
    void** inputs; // received external inputs
    int *inputs_rcv; // boolean map of active external inputs
    //int update;
    struct Cell1D *prev; // link to previous cell in the space
    struct Cell1D *next; // link to next cell in the space
} Cell1D;

typedef struct CS1D{
    Cell1D *M; // list of cells in the space
    int n_M; // number of cells in the space
    Cell1D **A; // array of active cells
    int max_A; // maximum number of posible active cells
    int n_A; // number of active cells
    double **hist; // history of states
    int n_hist; // number of steps recorded in the history
    int ncols; // size of the space (columns in 1D)
    int displayDelay; // delay to display animation in gnuplot
    int *neighborhood; // topology of the neighborhood
    int n_neighbors; // number of neighbors
    int n_inputs; // number of external inputs
    int plot_history; // boolean flag to display history or not
    int plot_animation; // boolean flag to display graphical animation or not
    int wrapped_borders; // space boundary configuration (0 none, 1 yes)
    FILE * gp; // gnuplot terminal file descriptor.
    void* (*dstate)(); // default state function
}CS1D;



int mod (int a, int b){
  int ret;
  
  if(b < 0) //you can check for b == 0 separately and do what you want
    return mod(-a, -b);   
  ret = a % b;
  if(ret < 0)
    ret+=b;
  return ret;
}

/* ###################################################*/
// Creates an empty 1D cellular space, using the received parameters.
void * CS1D_Create(int ncols, int *neighborhood, size_t n1, int n_inputs, int wrapped_borders, int plot_animation,int plot_range,int displayDelay,int plot_history, const char *path, const char *name){
    int i;
    CS1D *s;
    
    s = (CS1D *)malloc(sizeof(CS1D));
    s->M = NULL; // empty space -- (Cell1D *)malloc(ncols*sizeof(Cell1D));
    s->n_M = 0;
    s->max_A = ncols/2;
    s->A = (Cell1D **)malloc(s->max_A*sizeof(Cell1D*));//NULL; // no active cells
    s->n_A = 0;
    s->ncols = ncols;
    s->plot_history = plot_history;
    s->hist = NULL;
    s->n_hist = 0;
    s->n_neighbors = n1;
    s->n_inputs = n_inputs;
    s->neighborhood = (int *)malloc(sizeof(int)*n1);
    for(i=0;i<n1;i++)
        s->neighborhood[i] = neighborhood[i];
    s->plot_animation = plot_animation;
    s->wrapped_borders = wrapped_borders;
    s->displayDelay = displayDelay;
    s->dstate = NULL;
    if(plot_animation){
#ifdef _WIN32
        s->gp = _popen(path, "w");
#else
        s->gp = popen(path, "w");
#endif
        if(s->gp == NULL){
            fprintf(stderr, "Oops, I can't find %s.", path);
            exit(EXIT_FAILURE);
        }    
#ifdef _WIN32
        fprintf(s->gp, "set terminal wxt persist\n");
#else
        fprintf(s->gp, "set terminal x11 \n");
#endif
        fprintf(s->gp, "set title \"%s\" \n",name);
        fprintf(s->gp, "set yrange [%d:0] \n",2*ncols);
        if (plot_history)
            fprintf(s->gp, "set autoscale xy \n");
        fprintf(s->gp, "set cbrange [0:%d] \n",plot_range);
    }   
    return (void*)s;    
}


int CS1D_Delete(void* space){
    CS1D *s;
     int i;
    Cell1D *cell;

    s = (CS1D *)space;
    cell = s->M;
    for(i=0;i<s->n_M;i++){
	cell = s->M;
	s->M = cell->next;
	free(cell);
    }
    free(s->M);
    free(s->hist);
    //s->ncols = 0;
    free(s->neighborhood);
    if(s->plot_animation){
#ifdef _WIN32
	fprintf(s->gp, "pause mouse\n");
	//    ModelicaFormatMessage("Press any key to terminate (plot will close after pressing)...\n");
	//c = (char)getchar();
	_pclose(s->gp);
#else
	pclose(s->gp);
#endif
    }
    free(s);
    return 1;
}

int CS1D_SetDefault(void* space, void* (*defaultState)()){
    CS1D *s;
    s = (CS1D*)space;

    s->dstate = defaultState;
    return 1;
}


Cell1D* CS1D_FindCell(void* space, int pos, Cell1D* ref){
    CS1D *s;
    Cell1D *cell;
    int found;
    Cell1D *pre;
    
    s = (CS1D *)space;
    if (ref == NULL){ // no reference, start searching from the beginning
	cell = s->M;
	found = 0;
	while(cell != NULL && !found){
	    if(cell->pos == pos){
		found = 1;
	    }else{
		cell = cell->next;
	    }
	}	
    }else{ // start searching from given reference cell
	found = 0;
	cell = ref;
	pre = cell->prev;
	while ((cell != NULL || pre != NULL) && !found){
	    if (cell != NULL && cell->pos == pos){
		found = 1;
	    }else if (cell != NULL)
		cell = cell->next;	    
	    if (pre != NULL && pre->pos == pos){
		found = 2;
	    }else if (pre != NULL)
		pre = pre->prev;
	}
    }
	
    if(found == 1)
	return cell;
    else if (found == 2)
	return pre;
    else
	return NULL;
}

void CS1D_InsertCell(void* space, Cell1D* cell, int pos){
    CS1D *s;
    Cell1D *n;
    Cell1D *p;
    
    s = (CS1D *)space;
    n = s->M;
    p = s->M;
    s->n_M++;
    if (s->M == NULL){ // first cell in the space
	cell->prev = NULL;
	cell->next = NULL;
	s->M = cell;
    }else{
	if (s->M->pos > pos){ // goes before first cell in space
	    cell->prev = NULL;
	    cell->next = s->M;
	    s->M->prev = cell;
	    s->M = cell;
	}else{ // find place to insert cell
	    n = s->M->next;
	    p = s->M;
	    while(n != NULL && n->pos < pos){
		p = n;
		n = n->next;
	    }
	    // place found, insert cell
	    cell->prev = p;
	    cell->next = n;
	    if(n!=NULL)
		n->prev = cell;
	    p->next = cell;
	}
    }
    return;
}

Cell1D* CS1D_ActiveCell(void *space, Cell1D* cell, int pos){
    int k;
    CS1D *s;
    
    s = (CS1D *)space;
    if (cell == NULL){ // neighbor not present in the space
	// create new cell to insert into cellular space
	cell = (Cell1D *)malloc(sizeof(Cell1D));
	cell->cellstate = s->dstate(); // default state for neighbor
	cell->pos = pos;
	cell->prev = NULL;
	cell->next = NULL;
	cell->active = 0;
	//neighbor.update = 1;
	if(s->n_inputs > 0){
	    cell->inputs = (void**)malloc(sizeof(void*)*s->n_inputs);
	    cell->inputs_rcv = (int *)malloc(sizeof(int)*s->n_inputs);
	    for(k=0;k<s->n_inputs;k++){
		cell->inputs[k] = NULL;
		cell->inputs_rcv[k] = 0;
	    }
	}else{
	    cell->inputs = NULL;
	    cell->inputs_rcv = NULL;
	}
	//cell->neighbors = (Cell1D **)malloc(s->n_neighbors*sizeof(Cell1D *)); // array of pointers to neighbors
	cell->neighbors_present = (int *)malloc(s->n_neighbors*sizeof(int));
	for(k=0;k<s->n_neighbors;k++){
	    // check for wrapped boundaries.
	    if ((pos+s->neighborhood[k] < 0 || pos+s->neighborhood[k] > s->ncols-1) && !s->wrapped_borders) {
		cell->neighbors_present[k] = -1; // neighbor not present, outside row border and not wrapped
	    }else{ // neighbor present
		cell->neighbors_present[k] =  mod(pos+s->neighborhood[k],s->ncols);
		//neighbor = CS1D_FindCell(space,mod(pos+s->neighborhood[k],s->ncols));
		//CS1D_ActiveNeighbor(space,neighbor,mod(pos+s->neighborhood[k],s->ncols)); //neighbor.neighbors[k]->update = 1;
		//neighbor.neighbors[k] = neighbor; //&(s->M[mod(s->neighborhood[k]+i,ncols)]);
	    }         
	}
	CS1D_InsertCell(space,cell,pos); // insert new cell into space
    }
    
    if(!cell->active){
	cell->active=1;
	s->n_A++;
	if(s->n_A > s->max_A){
	    s->max_A = s->max_A*2;//+s->ncols/2;
	    s->A = (Cell1D **)realloc(s->A,s->max_A*sizeof(Cell1D*));
	}
	s->A[s->n_A-1] = cell;
    }
    return cell;
}


Cell1D* CS1D_Init(void* space, int modpos, void* (*initialState)()){
    int k,pos;
    CS1D *s;
    Cell1D *newcell;
    
    pos = modpos-1;
    s = (CS1D *)space;
    newcell = CS1D_FindCell(space,pos,NULL);
    if (newcell != NULL){ // cell already in space
	newcell->cellstate = initialState();
    }else{
	// create new cell to insert into cellular space
	newcell = (Cell1D *)malloc(sizeof(Cell1D));
	newcell->cellstate = initialState();
	newcell->pos = pos;
	newcell->prev = NULL;
	newcell->next = NULL;
	newcell->active = 0;
	//newcell.update = 1;
	if(s->n_inputs > 0){
	    newcell->inputs = (void**)malloc(sizeof(void*)*s->n_inputs);
	    newcell->inputs_rcv = (int *)malloc(sizeof(int)*s->n_inputs);
	    for(k=0;k<s->n_inputs;k++){
		newcell->inputs[k] = NULL;
		newcell->inputs_rcv[k] = 0;
	    }
	}else{
	    newcell->inputs = NULL;
	    newcell->inputs_rcv = NULL;
	}
	//newcell->neighbors = (Cell1D **)malloc(s->n_neighbors*sizeof(Cell1D *)); // array of pointers to neighbors
	newcell->neighbors_present = (int *)malloc(s->n_neighbors*sizeof(int));
	for(k=0;k<s->n_neighbors;k++){
	    // check for wrapped boundaries.
	    if ((pos+s->neighborhood[k] < 0 || pos+s->neighborhood[k] > s->ncols-1) && !s->wrapped_borders) {
		newcell->neighbors_present[k] = -1; // neighbor not present, outside row border and not wrapped
	    }else{ // neighbor present
		newcell->neighbors_present[k] = mod(pos+s->neighborhood[k],s->ncols);
		//neighbor = CS1D_FindCell(space,mod(pos+s->neighborhood[k],s->ncols));
		//newcell.neighbors[k] = neighbor; //&(s->M[mod(s->neighborhood[k]+i,ncols)]);
		//CS1D_ActiveCell(space,neighbor,mod(pos+s->neighborhood[k],s->ncols)); //newcell.neighbors[k]->update = 1;
	    }         
	}
	CS1D_InsertCell(space,newcell,pos); // insert new cell into space
    }
    CS1D_ActiveCell(space,newcell,pos); // set cell as active 
    return newcell;
}

void CS1D_Set(void* space, int pos, void* value){ // SETS CELL STATE
    Cell1D * cell;

    cell = CS1D_FindCell(space,pos,NULL);
    if (cell != NULL)
	cell->cellstate = value;
    CS1D_ActiveCell(space,cell,pos);
     
    return;
}


void CS1D_SetInput(void* space, int modpos, void* value, int input_id){ // SETS CELL INPUT
  Cell1D *cell;
  int pos;
  
  pos = modpos-1;
  cell = CS1D_FindCell(space,pos,NULL);
  cell = CS1D_ActiveCell(space,cell,pos);
  if (cell != NULL){
      if (cell->inputs[input_id-1] != NULL) // free previous input
	  free(cell->inputs[input_id-1]);
      cell->inputs[input_id-1] = value;
      cell->inputs_rcv[input_id-1] = 1;
      // CS1D_ActiveCell(space,cell,pos);
      //  s->M[pos-1].update = 1;
  }
  return;
}


void* CS1D_GetState(void* space, int modpos){
    Cell1D *cell;
    CS1D *s;
    int pos;
    
    s = (CS1D *)space;
    pos = modpos-1;
    cell = CS1D_FindCell(space,pos,NULL);
    if (cell == NULL) // cell not present in space (return default state)
	return s->dstate();
    else
	return cell->cellstate;
}



int CS1D_Size(void* space){
    CS1D *s;
    
    s = (CS1D *)space;
    return s->ncols;
}


void CS1D_Step(void* space, void* (*rule)(int*,void*,int,void**,int,int*,void**,int,int*),double (*display)(void*)){
    void** new_states;
    void* news;
    int i,j,k,nA;
    CS1D *s;
    void** neighbors;
    Cell1D** new_A;
    //int max_new_A;
    int new_n_A = 0;
    Cell1D* cell;
    void* auxstate;
    int* change;
    
    s = (CS1D *)space;
    
    // Store state and print current history
    if(s->plot_animation){
	fprintf(s->gp,"plot \'-\' matrix with image\n");
	if (s->plot_history){
	    s->n_hist++;
	    s->hist = (double**)realloc((double**)s->hist,sizeof(double*)*s->n_hist);
	    s->hist[s->n_hist-1] = (double*)malloc(sizeof(double)*s->ncols);
	    if (s->hist == NULL){
		//ModelicaFormatMessage("realloc failed!\n!");
		exit(-1);
	    }
	    // store current states in history
	    cell = s->M;    
	    for (i=0;i<s->ncols;i++){
		if (cell != NULL && cell->pos == i){
		    s->hist[s->n_hist-1][i] = display(cell->cellstate);
		    cell = cell->next;
		}else{
		    s->hist[s->n_hist-1][i] = display(NULL);
		}
	    }
	    // print history
	    for(i=0;i<s->n_hist;i++){
		for(j=0;j<s->ncols;j++){
		    fprintf(s->gp,"%f ",s->hist[i][j]);
		    //printf("%d ",s->hist[i][j]);
		}
		fprintf(s->gp,"\n");
		//printf("\n");
	    }
	} else{ // NO HISTORY, print first emtpy row to avoid gnuplot problem (min matrix plot of size 2x2)
	    for(j=0;j<s->ncols;j++){
		fprintf(s->gp,"0 ");
		//printf("%d ",s->hist[i][j]);
	    }       
	    fprintf(s->gp,"\n");
	}
    }
    
    // Set neighbors of current active cell also active.
    nA = s->n_A;
    for(i=0;i<nA;i++){
	//k = s->A[i]->pos;
	for(j=0;j<s->n_neighbors;j++){
	    if(s->A[i]->neighbors_present[j] > -1){
		CS1D_ActiveCell(space,CS1D_FindCell(space,s->A[i]->neighbors_present[j],s->A[i]),s->A[i]->neighbors_present[j]);
	    }
	}         
    }   
    
    // dynamic memory for neighbors
    neighbors = (void**)malloc(s->n_neighbors*sizeof(void*));
    
    // calculate and print new cellstates
    new_states = (void**)malloc(sizeof(void*)*s->n_A);  
    auxstate = s->dstate();
    //max_new_A = s->max_A;
    new_A = (Cell1D**)malloc(s->max_A*sizeof(Cell1D*));
    new_n_A = 0;
    change = (int *)malloc(s->ncols*sizeof(int));
    for(i=0;i<s->ncols;i++)
	change[i] = 0;
    for(i=0;i<s->n_A;i++){
	// join neighbor states
	for(k=0;k<s->n_neighbors;k++)
	    if (s->A[i]->neighbors_present[k] > -1){
		cell = CS1D_FindCell(space,s->A[i]->neighbors_present[k],s->A[i]); // find neighbor
		if (cell == NULL)
		    neighbors[k] = auxstate; // neighbor not found, using default state
		else
		    neighbors[k] = cell->cellstate; // neighbor found
	    }
	cell = s->A[i];
	// calculate new state using user defined rule function
	news = rule(&(change[cell->pos]), cell->cellstate,cell->pos,neighbors,s->n_neighbors,cell->neighbors_present,cell->inputs,s->n_inputs,cell->inputs_rcv);
	new_states[i] = news;
	// keep cell active if state changed
	if (change[cell->pos]){
	    cell->active=1;
	    new_n_A++;
	    if(new_n_A > s->max_A){
		s->max_A = s->max_A*2;
		new_A = (Cell1D **)realloc(new_A,s->max_A*sizeof(Cell1D*));
	    }
	    new_A[new_n_A-1] = cell;
	}else // otherwise, set as inactive
	    cell->active = 0;
	// deactivate inputs
	for (k=0;k<s->n_inputs;k++){
	    free(s->A[i]->inputs[k]);
	    s->A[i]->inputs[k] = NULL;
	    s->A[i]->inputs_rcv[k] = 0;
	}
    }
    
    // update cellstates
    for(i=0;i<s->n_A;i++){
	cell = s->A[i];
	//cell = CS1D_FindCell(space,s->A[i]->pos,NULL);
	if(cell != NULL && new_states[i] != NULL){
	    free(cell->cellstate);
	    cell->cellstate = new_states[i];
	    new_states[i] = NULL;
	}
    }
    
    if(s->plot_animation){   
	cell = s->M;
	for (i=0;i<s->ncols;i++){
	    if (cell != NULL && i == cell->pos){ // cell present in space, print state.
		fprintf(s->gp,"%f ",display(cell->cellstate));
		cell = cell->next;
	    }else{// cell not present, print default
		fprintf(s->gp,"%f ",display(NULL));
	    }
	}
    }
    
    if(s->plot_animation){
	fprintf(s->gp,"\n");
	fprintf(s->gp,"\n");
	fprintf(s->gp,"e\n");
	//printf("\n");
	//printf("\n");
	//printf("e\n");
#ifdef _WIN32
	Sleep(s->displayDelay/1000);
#else
	usleep(s->displayDelay);
#endif
	fflush(s->gp);
    }
    

    free(s->A);
    s->A = new_A;
    s->n_A = new_n_A;
    free(new_states);
    free(neighbors);
    free(change);
    return;
}

void CS1D_NSteps(int N,void* space, void* (*rule)(int*, void*,int,void**,int,int *,void**,int,int*),double (*display)(void*)){
    int i;
    
    for (i=0;i<N;i++)
	CS1D_Step(space,rule,display);
    
    return;
    
}

      
 
// ****************************************************
// 2D CELLULAR AUTOMATA
// ****************************************************


/* data type */

typedef struct Cell2D{
    void* cellstate; // state of the cell
    int row; // row in 2D space 
    int col; // column in 2D space
    int active; // boolean flag to set cell as active or not
    struct Cell2D **neighbors; // references to neighbors
    void** inputs; // received external inputs
    int *inputs_rcv; // boolean map of active external inputs
    //int update;
    struct Cell2D *prev; // link to previous cell in the same row
    struct Cell2D *next; // link to next cell in the same row
} Cell2D;

typedef struct CS2D{
    Cell2D **M; // matrix of cells in the space 
    int *n_M; // number of cells in the space
    Cell2D **A; // array of active cells
    int n_A; // number of active cells
    int max_A; // maximum number of possible active cells
    int ncols; // number of columns of the space
    int nrows; // number of rows of the space
    int lattice; // geometry of the cells in the lattice (0 = square, 1 = hexagonal)
    int **eneighborhood; // neighborhood for even rows in hexagonal lattice
    int displayDelay; // delay to display animation in gnuplot
    int **neighborhood; // topology of the neighborhood
    int n_neighbors; // number of neighbors
    int n_inputs; // number of inputs
    int plot_animation; // boolean flag to display graphical animation or not
    int wrapped_borders; // space boundary configuration (0 none wrapped, 1 wrapped north-south, 2 wrapped east-west, 3 all wrapped)
    FILE * gp; // gnuplot terminal file descriptor
}CS2D;


/* ###################################################*/

void* CS2D_Create(int nrows, int ncols, int lattice, int *neighborhood, size_t n1, size_t n2, int n_inputs, int wrapped_borders,int plot_animation,int plot_range,int displayDelay, const char *path,const char *name){
    CS2D *s;
    int i,j;
    int lneigh[12] = {-1,0,-1,1,0,-1,0,1,1,0,1,1}; // neighborhood for odd rows in hexagonal lattice
    int evenlneigh[12] = {-1,-1,-1,0,0,-1,0,1,1,-1,1,0}; // neighborhood for even rows in hexagonal lattice
    
    s = (CS2D *)malloc(sizeof(CS2D));
    s->M = (Cell2D **)malloc(nrows*sizeof(Cell2D*));// empty space - (Cell2D **)malloc(nrows*sizeof(Cell2D*));
    s->n_M = (int*)malloc(nrows*sizeof(int));
    for(i=0;i<nrows;i++){
	s->M[i] = NULL;
	s->n_M[i] = 0;    
    }
    s->max_A = ncols;
    s->A = (Cell2D**)malloc(s->max_A*sizeof(Cell2D*));
    s->n_A = 0;
    s->ncols = ncols;
    s->nrows = nrows;
    s->lattice = lattice;
    s->n_neighbors = n1;
    s->n_inputs = n_inputs;
    s->neighborhood = (int **)malloc(sizeof(int *)*n1);
    s->eneighborhood = (int **)malloc(sizeof(int *)*6); // hexagonal lattice always has 6 neighbors
    for(i=0;i<n1;i++){
        s->neighborhood[i] = (int *)malloc(sizeof(int)*n2);
    }
    for(i=0;i<6;i++){
	s->eneighborhood[i] = (int *)malloc(sizeof(int)*2);
    }
    j = 0;
    for(i=0;i<n1*n2;i+=2){
	if(lattice){ // hexagonal lattice
	    s->neighborhood[j][0] = lneigh[i];
	    s->neighborhood[j][1] = lneigh[i+1];
	    s->eneighborhood[j][0] = evenlneigh[i];
	    s->eneighborhood[j][1] = evenlneigh[i+1];
	    j++;
	}else{
	    s->neighborhood[j][0] = neighborhood[i];
	    s->neighborhood[j][1] = neighborhood[i+1];
	    j++;
	}
    }
    s->plot_animation = plot_animation;
    s->wrapped_borders = wrapped_borders;
    s->displayDelay = displayDelay;
    if (plot_animation){
#ifdef _WIN32
        s->gp = _popen(path, "w");
#else
        s->gp = popen(path, "w");
#endif
	
        if(s->gp == NULL){
            fprintf(stderr, "Oops, I can't find %s.", path);
            exit(EXIT_FAILURE);
        }    
#ifdef _WIN32
        fprintf(s->gp, "set terminal wxt persist \n");
#else
        fprintf(s->gp, "set terminal x11 \n");
#endif
        fprintf(s->gp, "set title \"%s\" \n",name);
        fprintf(s->gp, "set yrange [%d:0] \n",nrows);
        fprintf(s->gp, "set autoscale xy \n");
        fprintf(s->gp, "set cbrange [0:%d] \n",plot_range);
    }
    
    return (void *)s;
}




int CS2D_Delete(void* space){
  CS2D *s;
  Cell2D *cell;
  int i,j;
  
  s =(CS2D *)space;
  cell = s->M[0];
  for(i=0;i<s->nrows;i++){
      for(j=0;j<s->n_M[i];j++){
	  cell = s->M[i];
	  s->M[i] = cell->next;
	  free(cell);
      }
  }
  free(s->M);
  free(s->n_M);
  free(s->A);
  s->ncols = 0;
  s->nrows = 0;
  free(s->eneighborhood);
  free(s->neighborhood);
  if(s->plot_animation){
#ifdef _WIN32
    fprintf(s->gp, "pause mouse\n");
    //    ModelicaFormatMessage("Press any key to terminate (plots will close after pressing)...\n");
    //c = (char)getchar();
    _pclose(s->gp);
#else
      pclose(s->gp);
#endif
  }
  free(s);
  return 1;
}


Cell2D* CS2D_FindCell(void* space, int row, int col, Cell2D *ref){
    CS2D *s;
    Cell2D *cell;
    int found,i;
    Cell2D *pre;
    
    
    s = (CS2D *)space;
    cell = NULL;
    found = 0;
    if (s->M[row] == NULL) // no cells in row
	return NULL;
    if (ref == NULL){ // no reference, start searching from the beginning of the row
	cell = s->M[row];
	while(cell != NULL && !found){
	    if(cell->col == col){
		found = 1;
	    }else{
		cell = cell->next;
	    }
	}	
    }else{ // start searching from given reference cell
	// search in the neighborhood of the reference
	i = 0;
	if (ref->row == row){ // search from reference
	    cell = ref;
	}else{
	    while (i<s->n_neighbors){  // search from neighbor
		if(ref->neighbors[i] != NULL) // neighbor present
		    if (ref->neighbors[i]->row == row){
			cell = ref->neighbors[i];
			i = s->n_neighbors;
		    }
		i++;	
	    }
	    if(cell == NULL) // search from the beginning of the row
		cell = s->M[row];
	}
	pre = cell->prev;
	while ((cell != NULL || pre != NULL) && !found){
	    if (cell != NULL && cell->col == col){
		found = 1;
	    }else if (cell != NULL)
		cell = cell->next;	    
	    if (pre != NULL && pre->col == col){
		found = 2;
	    }else if (pre != NULL)
		pre = pre->prev;
	}
    }
    
    if(found == 1){
	return cell;
    }else if (found == 2){
	return pre;
    }else{
	return NULL;
    }
}    



void CS2D_InsertCell(void* space, Cell2D* cell, int row, int col){
    CS2D *s;
    Cell2D *n;
    Cell2D *p;
    
    s = (CS2D *)space;
    n = s->M[row];
    p = s->M[row];
    s->n_M[row]++;
    if (s->M[row] == NULL){ // first cell in the space
	cell->prev = NULL;
	cell->next = NULL;
	s->M[row] = cell;
    }else{
	if (s->M[row]->col > col){ // goes before first cell in space
	    cell->prev = NULL;
	    cell->next = s->M[row];
	    s->M[row]->prev = cell;
	    s->M[row] = cell;
	}else{ // find place to insert cell
	    n = s->M[row]->next;
	    p = s->M[row];
	    while(n != NULL && n->col < col){
		p = n;
		n = n->next;
	    }
	    // place found, insert cell
	    cell->prev = p;
	    cell->next = n;
	    if(n!=NULL)
		n->prev = cell;
	    p->next = cell;
	}
    }
    return;
}

Cell2D* CS2D_ActiveCell(void *space, Cell2D* cell){
    int i,k;
    CS2D *s;
    
    s = (CS2D *)space;
    if(!cell->active){
	cell->active=1;
	s->n_A++;
	if(s->n_A > s->max_A){
	    s->max_A = s->max_A*2;
	    s->A = (Cell2D **)realloc(s->A,s->max_A*sizeof(Cell2D*));
	}s->A[s->n_A-1] = cell;
    }
    return cell;
}

Cell2D* CS2D_Init(void* space,int modrow, int modcol, void* (*initialState)()){
    int i,k,row,col;
    CS2D *s;
    Cell2D *newcell;

    row = modrow-1;
    col = modcol-1;
    s = (CS2D *)space;
    newcell = CS2D_FindCell(space,row,col,NULL);
    newcell->cellstate = initialState();
    CS2D_ActiveCell(space,newcell);
    
    return newcell;
}

void CS2D_InitDefault(void *space, void* (*defaultState)()){
    int i,j,k;
    int rows,cols;
    CS2D *s;
    Cell2D *newcell;
    int nrow,ncol,en;
 
    s = (CS2D *)space;
    rows = s->nrows;
    cols = s->ncols;

    for(i=0;i<rows;i++){
	for(j=0;j<cols;j++){
	    // create new cell to insert into cellular space
	    newcell = (Cell2D *)malloc(sizeof(Cell2D));
	    newcell->cellstate = defaultState(); // default state for neighbor
	    newcell->row = i;
	    newcell->col = j;
	    newcell->prev = NULL;
	    newcell->next = NULL;
	    newcell->active = 0;
	    //neighbor.update = 1;
	    if(s->n_inputs > 0){
		newcell->inputs = (void**)malloc(sizeof(void*)*s->n_inputs);
		newcell->inputs_rcv = (int *)malloc(sizeof(int)*s->n_inputs);
		for(k=0;k<s->n_inputs;k++){
		    newcell->inputs[k] = NULL;
		    newcell->inputs_rcv[k] = 0;
		}
	    }else{
		newcell->inputs = NULL;
		newcell->inputs_rcv = NULL;
	    }
	    newcell->neighbors = (Cell2D **)malloc(s->n_neighbors*sizeof(Cell2D *)); // array of pointers to neighbors
	    for(k=0;k<s->n_neighbors;k++)
		newcell->neighbors[k] = NULL;
	    CS2D_InsertCell(space,newcell,i,j); // insert new cell into space
	}
    }

    
    // once all the cells are created, assign neighbors
    for(i=0;i<rows;i++){
	for(j=0;j<cols;j++){
	    newcell = CS2D_FindCell(space,i,j,NULL);
	    for(k=0;k<s->n_neighbors;k++){
		if (s->lattice && (i % 2 == 0)){ // hexagonal lattice and even row
		    if ((i+s->eneighborhood[k][0] < 0 || i+s->eneighborhood[k][0] > s->nrows-1) && (s->wrapped_borders == 0 || s->wrapped_borders == 2)) {
			newcell->neighbors[k] = NULL; // neighbor not present, outside row border and not wrapped
		    }else if ((j+s->eneighborhood[k][1] < 0 || j+s->eneighborhood[k][1] > s->ncols-1) && (s->wrapped_borders == 0 || s->wrapped_borders == 1)) {
			newcell->neighbors[k] = NULL; // neighbor not present, outside column border and not wrapped
		    }else{ // neighbor present
			nrow = mod(i+s->eneighborhood[k][0],s->nrows);
			ncol = mod(j+s->eneighborhood[k][1],s->ncols);
			newcell->neighbors[k] = CS2D_FindCell(space,nrow,ncol,newcell);
		    }
		}else{ //square lattice or odd row
		    // check for wrapped boundaries
		    if ((i+s->neighborhood[k][0] < 0 || i+s->neighborhood[k][0] > s->nrows-1) && (s->wrapped_borders == 0 || s->wrapped_borders == 2)) {
			newcell->neighbors[k] = NULL; // neighbor not present, outside row border and not wrapped
		    }else if ((j+s->neighborhood[k][1] < 0 || j+s->neighborhood[k][1] > s->ncols-1) && (s->wrapped_borders == 0 || s->wrapped_borders == 1)) {
			newcell->neighbors[k] = NULL; // neighbor not present, outside column border and not wrapped
		    }else{ // neighbor present
			en = s->neighborhood[k][0];
			nrow = mod(i+s->neighborhood[k][0],s->nrows);
			en = s->neighborhood[k][1];
			ncol = mod(j+s->neighborhood[k][1],s->ncols);
			newcell->neighbors[k] = CS2D_FindCell(space,nrow,ncol,newcell);
		    }
		}
	    }
	}
    }
    return;
}


void CS2D_SetState(void* space, int modrow, int modcol,void* newstate){
    int row,col;
    Cell2D *cell;
    CS2D* s;
    
    s = (CS2D*)space;
    row = modrow -1;
    col = modcol -1;
    cell = CS2D_FindCell(space,row,col,NULL);
    cell->cellstate = newstate;
    CS2D_ActiveCell(space,cell);
    
    return;
}


void CS2D_SetInput(void* space, int modrow, int modcol, void* value, int input_id){
  Cell2D *cell;
  int row,col;

  row = modrow-1;
  col = modcol-1;
  cell = CS2D_FindCell(space,row,col,NULL);
  cell = CS2D_ActiveCell(space,cell);
  if (cell != NULL){
      if (cell->inputs[input_id-1] != NULL) // free previous input
	  free(cell->inputs[input_id-1]);
      cell->inputs[input_id-1] = value;
      cell->inputs_rcv[input_id-1] = 1;
  }
  return;
}


void *CS2D_GetState(void* space, int modrow, int modcol){
  CS2D *s;
  int row,col;
  Cell2D *cell;

  row = modrow-1;
  col = modcol-1;
  s = (CS2D *)space;
  cell = CS2D_FindCell(space,row,col,NULL);
  return cell->cellstate;
}


int CS2D_SizeRows(void* space){
  CS2D *s;
  
  s =(CS2D *)space;
  return s->nrows;
}

int CS2D_SizeCols(void* space){
  CS2D *s;
  
  s =(CS2D *)space;
  return s->ncols;
}


int CS2D_Step(void* space,void* (*rule)(int*,void*,int,int,void**,int,void**,int,int*),double (*display)(void*)){
    void **new_states;
    void* news;
    int i,k,nA;
    CS2D *s;
    void **neighbors;
    Cell2D** new_A;
    //int max_new_A;
    int new_n_A = 0;
    Cell2D* cell;
    //cell_list* lchange;
    int change;
    int active;
    
    struct timeval  tv1, tv2;
    gettimeofday(&tv1, NULL);



    s = (CS2D *)space;
    
    // Set neighbors of current active cell also active.
    nA = s->n_A;
    for(i=0;i<nA;i++){
	for(k=0;k<s->n_neighbors;k++){
	    if(s->A[i]->neighbors[k] != NULL){
		CS2D_ActiveCell(space,s->A[i]->neighbors[k]);
	    }
	}         
    }  
    
    // dynamic memory for the state of neighbors
    neighbors = (void **)malloc(sizeof(void *)*s->n_neighbors);
    
    // calculate new cellstates
    new_states = (void**)malloc(sizeof(void*)*s->n_A);  
    //max_new_A = s->max_A;
    new_A = (Cell2D**)malloc(s->max_A*sizeof(Cell2D*));
    new_n_A = 0;
    //ModelicaFormatMessage("Number Active %d\n",s->n_A);
    active = s->n_A;
    for(i=0;i<s->n_A;i++){
	// join neighbors states
	for(k=0;k<s->n_neighbors;k++){
	    if (s->A[i]->neighbors[k] != NULL){ // neighbor present
		neighbors[k] = s->A[i]->neighbors[k]->cellstate;
	    }else{
		neighbors[k] = NULL; // neighbor not present
	    }
	}
	cell = s->A[i];
	// calculate new state using user defined rule function
	//	ModelicaFormatMessage("STEP [%d,%d]\n",cell->row,cell->col);
	news = rule(&change, cell->cellstate,cell->row,cell->col,neighbors,s->n_neighbors,cell->inputs,s->n_inputs,cell->inputs_rcv);
	new_states[i] = news;
	// keep cell active if state changed
	if (change){
	    cell->active=1;
	    new_n_A++;
	    if (new_n_A > s->max_A){
		s->max_A = s->max_A*2;
		new_A = (Cell2D **)realloc(new_A,s->max_A*sizeof(Cell2D*));
	    }
	    new_A[new_n_A-1] = cell;
	}else // otherwise, set as inactive
	    cell->active = 0;
	// deactivate inputs
	for (k=0;k<s->n_inputs;k++){
	    free(s->A[i]->inputs[k]);
	    s->A[i]->inputs[k] = NULL;
	    s->A[i]->inputs_rcv[k] = 0;
	}
    }

    
    // update cellstates
    for(i=0;i<s->n_A;i++){
	cell = s->A[i];
	if(cell != NULL && new_states[i] != NULL){
	    free(cell->cellstate);
	    cell->cellstate = new_states[i];
	    new_states[i] = NULL;
	}
    }
    
    // plot animation for new states
    if(s->plot_animation){
	fprintf(s->gp,"plot \'-\' matrix with image\n");
	for (i=0;i<s->nrows;i++){
	    cell = s->M[i];
	    for(k=0;k<s->ncols;k++){
		if (cell != NULL && k == cell->col){ // cell present in space, print state.
		    fprintf(s->gp,"%f ",display(cell->cellstate));
		    cell = cell->next;
		}else{// cell not present, print default
		    fprintf(s->gp,"%f ",display(NULL));
		}
	    }
	    fprintf(s->gp,"\n ");
	}
    }
    
    if(s->plot_animation){
	//	fprintf(s->gp,"\n");
	fprintf(s->gp,"\n");
	fprintf(s->gp,"e\n");
	//printf("\n");
	//printf("\n");
	//printf("e\n");
#ifdef _WIN32
	Sleep(s->displayDelay/1000);
#else
	usleep(s->displayDelay);
#endif
	fflush(s->gp);
    }
    
    
    free(s->A);
    s->A = new_A;
    s->n_A = new_n_A;
    free(new_states);
    free(neighbors);
    //ModelicaFormatMessage("ACTIVE = %d\n",active);

    gettimeofday(&tv2, NULL);
    
    printf ("%f\n",
	    (double) (tv2.tv_usec - tv1.tv_usec) / 1000000 +
	    (double) (tv2.tv_sec - tv1.tv_sec));
    
    return active;
}

void CS2D_NSteps(int N, void* space, void * (*rule)(int*, void*,int,int,void**,int,void**,int,int*),double (*display)(void*)){
    int i;
  
  for (i=0;i<N;i++)
      CS2D_Step(space,rule,display);
  
  return;
}




#endif


