/**********************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the Modelica License 2
* Copyright © 2015, Victorino Sanz.
* 
***********************************************************/

#ifndef C17HPP
#define C17HPP

#include <CellularAutomataLib.c>


/****************************************/
/* LGCA MODELS (HPP) FOR ARGESIM C17 BENCHMARK      */
/****************************************/

//**************************************
// Cell state
// User may redefine the state of the cell as desired
typedef struct HPPState{
    // particles values 0=empty, 1= susceptible, 2= infected, 3= recovered
    int n; // north particle 
    int e; // east particle
    int w; // west particle
    int s; // south particle
}HPPState;



//**************************************
// CELLULAR SPACE FUNCTIONS

// Function that allocates memory and sets the default state for the cells
void * HPPDefaultState(){
    HPPState *s;
    s = (HPPState *)malloc(sizeof(HPPState));
    
    //set the default values for state variables
    s->n = 0;
    s->e = 0;
    s->w = 0;
    s->s = 0;
    return (void *)s;
}

int HPPDefault(void* space){
    void*  (*defaultState)() = &HPPDefaultState;
    CS2D_InitDefault(space, defaultState);
    return 1;
    
}

// Function that describes the state of initialized cells (non-initialized cells will remain in the default state)
void* HPPInitialState(){
    HPPState *s;
    s = (HPPState *)malloc(sizeof(HPPState));
    
    //set the default values for state variables
    s->n = 0;
    s->e = 1;
    s->w = 1;
    s->s = 0;
    return (void *)s;
}

// initializing function for the selected cell
void HPPInitial(void* space, int row, int col){
    void*  (*initialState)() = &HPPInitialState;
    CS2D_Init(space, row, col, initialState);
    return;
}

void* HPPRandomInitialState(){
    HPPState *s;
    s = (HPPState *)malloc(sizeof(HPPState));
    
    //set the default values for state variables
    s->n = rand() % 2;
    s->e = rand() % 2;
    s->w = rand() % 2;
    s->s = rand() % 2;
    return (void *)s;
}

void HPPRandomInitial(void* space, int row, int col){
    int i,j;
    void*  (*initialState)() = &HPPRandomInitialState;
    srand(time(NULL));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++)
	    if (rand() % 2)
		CS2D_Init(space, i, j, initialState);
    return;
}

void HPPSIRInitial(void* space, int row, int col){
    int i,j,n,k,pos;
    void*  (*defaultState)() = &HPPDefaultState;
    int iniS = 16000;
    int iniI = 100;
    int iniR = 0;
    HPPState *s;
    
    srand(time(NULL));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++)
	    CS2D_Init(space,i,j,defaultState);

 
    n = 0;
    while(n < iniS){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k = 1;
	pos = rand()%4;
	while(k<5){
	    if(pos == 0 && s->n == 0){
		s->n = 1;
		break;
	    }else if (pos == 1 && s->w == 0){
		s->w = 1;
		break;
	    }else if (pos == 2 && s->e == 0){
		s->e = 1;
		break;
	    }else if (pos == 3 && s->s == 0){
		s->s = 1;
		break;
	    }else{
		pos = (pos+1)%4;
		k++;
	    }
	}
	if (k<5)
	    n++;
	CS2D_SetState(space,i+1,j+1,s);
    }

   n = 0;
    while(n < iniI){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k = 1;
	pos = rand()%4;
	while(k<5){
	    if(pos == 0 && s->n == 0){
		s->n = 2;
		break;
	    }else if (pos == 1 && s->w == 0){
		s->w = 2;
		break;
	    }else if (pos == 2 && s->e == 0){
		s->e = 2;
		break;
	    }else if (pos == 3 && s->s == 0){
		s->s = 2;
		break;
	    }else{
		pos = (pos+1)%4;
		k++;
	    }
	}
	if (k<5)
	    n++;
	CS2D_SetState(space,i+1,j+1,s);
    }

   n = 0;
   while(n < iniR){
       i = rand()%row;
       j = rand()%col;
       s = CS2D_GetState(space,i+1,j+1);
       k = 1;
       pos = rand()%4;
       while(k<5){
	   if(pos == 0 && s->n == 1){
	       s->n = 3;
	       break;
	   }else if (pos == 1 && s->w == 1){
	       s->w = 3;
	       break;
	   }else if (pos == 2 && s->e == 1){
	       s->e = 3;
	       break;
	   }else if (pos == 3 && s->s == 1){
	       s->s = 3;
	       break;
	   }else{
	       pos = (pos+1)%4;
	       k++;
	   }
       }
       if (k<5)
	   n++;
       CS2D_SetState(space,i+1,j+1,s);
   }
   return;
}


#ifndef C17FHP
double uniform(double a, double b)
{
    return rand() / (RAND_MAX + 1.0) * (b - a) + a;
}
#endif

//***************************************************
// RULE FUNCTION
void *HPPtransition(int *change, void* cellstate, int row, int col, void** neighbors, int n_neighbors, void** inputs, int n_inputs, int *inputs_rcv){
    HPPState *out;
    HPPState *aux;
    HPPState *cs;
    HPPState **ne;
    HPPState **in;
    double a = 0.2; // recovery rate
    double r = 0.6; // infection rate
    int i,I;
    double u;

    cs = (HPPState *)cellstate;
    ne = (HPPState **)neighbors;
    in = (HPPState **)inputs;
    out = (HPPState *)malloc(sizeof(HPPState));
    aux = (HPPState *)malloc(sizeof(HPPState));
    
    // Order declared for neighbors (W,S,E,N)

    // PROPAGATION
    out->w = 0;
    out->s = 0;
    out->e = 0;
    out->n = 0;
    if(neighbors[0] != NULL){ // N
	out->s = ne[0]->s;
    }
    if(neighbors[1] != NULL){ // W
	out->e = ne[1]->e;
    }
    if(neighbors[2] != NULL){ // E
	out->w = ne[2]->w;
    }
    if(neighbors[3] != NULL){ // S
	out->n = ne[3]->n;
    }
    
    //srand(time(NULL));
    // COLLISION
    if (out->w && out->e && !out->n && !out->s){
	//ModelicaFormatMessage("Collision WE\n");
	// predefined orientation for collision
	out->n = out->w;
	out->s = out->e;
	// random orientation for collision
	//ModelicaFormatMessage("Rand 50% = %d\n",rand()%2);
	/*if (rand() % 2){
	    out->n = out->w;
	    out->s = out->e;
	}else{
	    out->s = out->w;
	    out->n = out->e;
	    }*/
	out->w = 0;
	out->e = 0;
    }else if (out->n && out->s && !out->w && !out->e){
	//ModelicaFormatMessage("Collision NS\n");
	// predefined orientation for collision
	out->w = out->n;
	out->e = out->s;
	// random orientation for collision
	/*if (rand() % 2){
	    out->w = out->n;
	    out->e = out->s;
	}else{
	    out->e = out->n;
	    out->w = out->s;
	    }*/
	out->n = 0;
	out->s = 0;	
    }
    
    // DISEASE SPREAD
    // Infection
    I = 0;
    if(out->n == 2)
	I++;
    if(out->w == 2)
	I++;
    if(out->e == 2)
	I++;
    if(out->s == 2)
	I++;
    u = uniform(0,1);
    if(I && out->n == 1 && u <= r)//(1-pow(1-r,I)))
	out->n = 2;
    u = uniform(0,1);
    if(I && out->w == 1 && u <= r)//(1-pow(1-r,I)))
	out->w = 2;
    u = uniform(0,1);
    if(I && out->e == 1 && u <= r)//(1-pow(1-r,I)))
	out->e = 2;
    u = uniform(0,1);
    if(I && out->s == 1 && u <= r)//(1-pow(1-r,I)))
	out->s = 2;
    
    // Recovery    
    u = uniform(0,1);
    if(out->n == 2 && u <= a ){
	out->n = 3;
    }
    u = uniform(0,1);
    if(out->w == 2 && u <= a ){
	out->w = 3;
    }
    u = uniform(0,1);
    if(out->e == 2 && u <= a ){
	out->e = 3;
    }
    u = uniform(0,1);
    if(out->s == 2 && u <= a )
	out->s = 3;
    
    
   *change = 0;
   if(cs->n != out->n)
       *change = 1;
   if(cs->s != out->s)
       *change = 1;
   if(cs->e != out->e)
       *change = 1;
   if(cs->w != out->w)
       *change = 1;

    return (void *)out;
}

// Function that returns the value to display in the graphical animation
double HPPDisplay(void *cell){
    HPPState *s;
    double out;

    s = (HPPState *)cell;
    // set out to the desired value calculated from the state variables
    out = 0;
    if (cell == NULL)
	return 0;
    else{
	if(s->n == 2)
	    out++;
	if(s->w == 2)
	    out++;
	if(s->e == 2)
	    out++;
	if(s->s == 2)
	    out++;
    }
    return out;
}

int HPPStep(void* space){
    void* (*rule)(int*,void*,int,int,void**,int,void**,int,int*) = &HPPtransition;
    double (*display)(void*) = &HPPDisplay;
    
    CS2D_Step(space,rule,display);
    return 1;
}


//**************************************
// INTERFACE FUNCTIONS

// function to convert the state of a cell into a double value
double HPPOutputS(void* space, int row, int col){
    double out;
    HPPState* s;
    int i;
    
    s = (HPPState *)CS2D_GetState(space,row,col);
    
    out = 0;
    if (s->n == 1)
	out++;
    if (s->w == 1)
	out++;
    if (s->e == 1)
	out++;
    if (s->s == 1)
	out++;
    
    return out;
}

// function to convert the state of a cell into a double value
double HPPOutputI(void* space, int row, int col){
    double out;
    HPPState* s;
    int i;
    
    s = (HPPState *)CS2D_GetState(space,row,col);
    out = 0;
    if (s->n == 2)
	out++;
    if (s->w == 2)
	out++;
    if (s->e == 2)
	out++;
    if (s->s == 2)
	out++;
	
    return out;
}

// function to convert the state of a cell into a double value
double HPPOutputR(void* space, int row, int col){
    double out;
    HPPState* s;
    int i;
    
    s = (HPPState *)CS2D_GetState(space,row,col);
    out = 0;
    if (s->n == 3)
	out++;
    if (s->w == 3)
	out++;
    if (s->e == 3)
	out++;
    if (s->s == 3)
	out++;
    
    return out;
}


#endif
